#!/usr/local/bin/ruby

require "socket"
require "fcntl"


$CVSROOT = ENV["CVSROOT"]

$port = 2401
$com = "cvs"
$arg = ["-f", "--allow-root=#{$CVSROOT}", "pserver"]


# become daemon.
pid = fork
if pid
  # on fork failure.
  raise "fork failed." if pid < 0

  # parent exit immediately to pass the child init(1).
  exit(0)
end
Process.setsid


# close standard I/O port.
f = File.open("/dev/null", "r+")
$stdin.reopen(f)
$stdout.reopen(f)
$stderr.reopen(f)
f.close


# prepare log.
$logfile = "#{ENV['HOME']}/tmp/log.#{$$}"
$log = File.open($logfile, "w")
$log.sync = true


ss = TCPServer.new($port)
Thread.abort_on_exception = true	# just a charm :-)


while
  ns = ss.accept
  peer = ns.peeraddr
  $log.puts "accepted #{peer[2]}: #{peer[3]}"

  Thread.new do
    s = ns
    c = peer
    $log.puts "forked for #{c[2]}: #{c[3]}"

    pid = Process.fork do
      # clear close-on-exec flag.
      s.fcntl(Fcntl::F_SETFD, 0)

      # dup socket to standard I/O port.
      $stdin.reopen(s)
      $stdout.reopen(s)
      $stderr.reopen(s)
      s.close

      # exec external command.
      exec $com, *$arg
    end

    # parent wait for exit of child server.
    s.close
    Process.waitpid(pid, 0) if pid

    $log.puts "#{c[2]}: #{c[3]} leaved"
  end
end
